import BaseFoundation from '../base/foundation';
import { keyToCode } from './constants';
export default class HotKeysFoundation extends BaseFoundation {
  constructor(adapter) {
    super(Object.assign({}, adapter));
    this.handleKeyDown = event => {
      const disabled = this.getProps().disabled;
      if (disabled) {
        return;
      }
      const hotKeys = this.getProps().hotKeys;
      let allModifier = new Array(4).fill(false); // Meta Shift Alt Ctrl
      let clickedModifier = [event.metaKey, event.shiftKey, event.altKey, event.ctrlKey];
      const keysPressed = hotKeys === null || hotKeys === void 0 ? void 0 : hotKeys.map(key => {
        if (key === "Meta") {
          allModifier[0] = true;
          return event.metaKey;
        } else if (key === "Shift") {
          allModifier[1] = true;
          return event.shiftKey;
        } else if (key === "Alt") {
          allModifier[2] = true;
          return event.altKey;
        } else if (key === "Control") {
          allModifier[3] = true;
          return event.ctrlKey;
        }
        return event.code === keyToCode(key);
      });
      if (!allModifier.every((value, index) => value === clickedModifier[index])) {
        return;
      }
      if (keysPressed.every(Boolean)) {
        event.preventDefault();
        this.handleClick();
        return;
      }
    };
  }
  init() {
    // init Listener
    const target = this._adapter.getListenerTarget();
    target === null || target === void 0 ? void 0 : target.addEventListener('keydown', this.handleKeyDown);
  }
  handleClick() {
    this._adapter.notifyClick();
  }
  destroy() {
    // remove Listener
    const target = this._adapter.getListenerTarget();
    target === null || target === void 0 ? void 0 : target.removeEventListener('keydown', this.handleKeyDown);
  }
}